#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Reflection;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [StorableClass]
  [Item("FunctionSymbol", "Symbol that wraps a user-defined binary operation.")]
  public sealed class FunctionSymbol : Symbol {
    private int minimumArity = 2;
    private int maximumArity = 2;

    public override int MinimumArity {
      get { return minimumArity; }
    }
    public override int MaximumArity {
      get { return maximumArity; }
    }

    //[Storable]
    private MethodInfo methodInfo;
    public MethodInfo MethodInfo {
      get { return methodInfo; }
      private set { methodInfo = value; }
    }

    [StorableConstructor]
    private FunctionSymbol(bool deserializing) : base(deserializing) { }

    private FunctionSymbol(FunctionSymbol original, Cloner cloner) : base(original, cloner) {
      this.MethodInfo = original.MethodInfo; // symbols are cached so we also cache the MethodInfo in order to optimize the performance of the interpreter
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new FunctionSymbol(this, cloner);
    }

    public FunctionSymbol(string name, Func<double, double> f) : base(name, "Symbol that wraps a user-defined unary operation.") {
      MethodInfo = f.GetMethodInfo();
      minimumArity = maximumArity = 1;
    }

    public FunctionSymbol(string name, Func<double, double, double> f) : base(name, "Symbol that wraps a user-defined binary operation.") {
      MethodInfo = f.GetMethodInfo();
      minimumArity = maximumArity = 2;
    }

    public FunctionSymbol(string name, Func<double, double, double, double> f) : base(name, "Symbol that wraps a user-defined tertiary operation.") {
      MethodInfo = f.GetMethodInfo();
      minimumArity = maximumArity = 3;
    }

    public FunctionSymbol(string name, Func<double, double, double, double, double> f) : base(name, "Symbol that wraps a user-defined quaternary operation.") {
      MethodInfo = f.GetMethodInfo();
      minimumArity = maximumArity = 4;
    }
  }
}
